import java.util.ArrayList;
import java.util.Scanner;

public class ScannerExtension {

    /* Design Decision: This extension file has been designed so that all scanner logic is held in one place
     and future developers know exactly where to look if there is something wrong with a scanner.

    The middle two scanners are needed to achieve specific goals but have been designed to have a level of
    reusability and been given method names to make it clear what goal they achieve.

    The last scanner is very specific to the method which calls it but despite the cohesion it has been placed
    in this file so that all the scanner logic for the program stays together.

    I have added these Scanner extensions so that the user is unable to crash the program by typing incorrect
    information, thus creating a better user experience and making the program more resilient.

    These Self-Contained methods are static so that you do not need to create a new class everytime we need to use one.
     */

    public static Scanner scanner = new Scanner(System.in);

    /**
     * Takes in a string from the scanner and converts it to lower case
     * @return lower case version of the string entered into the scanner
     */
    public static String scanStringToLowercase() {
        String nextString = scanner.next();
        nextString = nextString.toLowerCase();
        return nextString;
    }

    /*
    Known Edge Case: For the below method if you type an incorrect int followed by a letter it crashes
    the program. This will be fixed in future editions.
     */

    /**
     * Extends Scanner to include error handling so that a user can only pick an integer in the given range
     * <p>
     * Method will keep asking for a user choice until user has picked a valid integer
     * @param totalChoices User can pick a number from 1 - totalChoices (inclusive).
     * @return Users integer choice
     */
    public static int scanForUserIntChoice(int totalChoices) {
        boolean correctEntry = false;
        // Prevents a user from entering a non-integer and asks them to pick again
        while (scanner.hasNextInt() == false) {
            scanner.next();
            System.out.println("Incorrect entry!  Please choose from one of the options above.");
        }
        int nextInt = scanner.nextInt();
        while (!correctEntry) {
            // Scanner will keep looping until the user has entered a valid choice
            if(nextInt <= totalChoices && nextInt > 0) {
                correctEntry = true;
            } else {
                System.out.println("Incorrect entry!  Please choose from one of the options above.");
                nextInt = scanner.nextInt();
            }
        }
        // Return a valid choice which has been error handled so program won't crash
        return nextInt;
    }


    /**
     * User can choose between y or n, this will be converted to lowerCase so Y and N work as well
     * <p>
     * Method will keep asking for a user choice until they pick a correctEntry
     * @return y or n
     */
    public static String chooseYorN() {
        boolean correctEntry = false;
        String nextString = scanner.next();
        /* Convert user input to lowercase and check if they have entered y or n,
        Loop will continue until user makes one of those two choices
         */
        nextString = nextString.toLowerCase();
        while (!correctEntry) {
            if(nextString.equals("y") || nextString.equals("n")) {
                correctEntry = true;
            } else {
                System.out.println("Incorrect entry!  Please choose 'y' or 'n'.");
                nextString = scanner.next();
                nextString = nextString.toLowerCase();
            }
        }
        return nextString;
    }

    /**
     *User can type in all to see all users data or theirs own name to check their data
     *<p>
     *If the name the user inputs doesn't match any data in the UserCompletionData then user can try another
     *name or press x to exit and go back to the main menu
     * @param users UserStorage object for user input from scanner to be checked against
     */
    public static void viewUserData(UsersStorage users) {
        String nextString = scanner.next();
        nextString = nextString.toLowerCase();
        // Checks if string equals all, if so it prints out all user data
        if(nextString.equals("all")) {
            users.printAllUsersData();
        } else {
            /*
            If the user didn't type all then the input will be checked to see if the username has data attached to it
            Loop will keep going until a correct username is given or the user decides to exit by pressing x.
             */
            boolean correctEntry = false;
            while (!correctEntry){
                int userExists = users.checkIfUserExists(nextString);
                if(userExists == -1) {
                    System.out.println("This user doesn't exist.  Please enter another username or 'x' to exit.");
                    nextString = scanner.next();
                    nextString = nextString.toLowerCase();
                    if(nextString.equals("x")) {
                        correctEntry = true;
                    }
                } else {
                    ArrayList<Lesson>  userRecord= users.getUsers().get(userExists).getUserRecord();
                    System.out.println(nextString + ":");
                    for(int i = 0; i < userRecord.size(); i++) {
                        System.out.println(users.printOutUserCompletionDataForLesson(userRecord.get(i)));
                    }
                    correctEntry = true;
                }
            }
        }
    }
}
